@@ -13,7 +13,7 @@ use MooX::Types::MooseLike::Numeric qw(PositiveInt);
 use Try::Tiny;
 use namespace::autoclean;
 
-our $VERSION = '1.003';
+our $VERSION = '1.004';
 
 # plural_ref e.g. ru
 # The key is the plural form 0, 1 or 2.
@@ -386,7 +386,9 @@ sub translate { ## no critic (ExcessComplexity)
         }
         ## no critic (ComplexRegexes)
         m{
-            \A \QAPI error\E
+            \A \QAPI error: \E
+            |
+            \A \QByte limit exceeded, \E
             |
             \A (?: Before | After )
             (?: \Q paragraph\E | \Q line\E )?
@@ -446,7 +448,7 @@ sub _translate_named {
     my ( $self, $entry_ref, $po ) = @_;
 
     my $msgid = $self->_encode_named( $entry_ref->{msgid} );
-    my $msgstr = $self->translate_any_msgid($msgid);
+    my $msgstr = $self->translate_any_msgid($msgid, 1);
     $msgstr = $self->_decode_named($msgstr);
     $po->msgstr( $entry_ref->{encode_obj}->encode($msgstr) );
 
@@ -468,7 +470,7 @@ sub _translate_named_plural {
                 ? ( $msgid_plural, $self->_plural_ref->{$index} )
                 : $msgid,
         );
-        my $any_msgstr = $self->translate_any_msgid($any_msgid);
+        my $any_msgstr = $self->translate_any_msgid($any_msgid, 1);
         $any_msgstr = $self->_decode_named($any_msgstr);
         $po->msgstr_n->{$index}
            = $po->quote( $entry_ref->{encode_obj}->encode($any_msgstr) );
@@ -483,7 +485,7 @@ sub _encode_gettext_inner { ## no critic (ManyArgs)
     $self->_gettext_ref->{$inner} ||= [
         map {
             ( defined && length )
-            ? $self->translate_any_msgid($_)
+            ? $self->translate_any_msgid($_, 1)
             : undef;
         } $singular, $plural, $zero
     ];
@@ -561,7 +563,7 @@ sub _translate_gettext {
 
     $self->_clear_gettext_ref;
     my $msgid = $self->_encode_gettext( $entry_ref->{msgid} );
-    my $msgstr = $self->translate_any_msgid($msgid);
+    my $msgstr = $self->translate_any_msgid($msgid, 1);
     $msgstr = $self->_decode_gettext($msgstr);
     $po->msgstr( $entry_ref->{encode_obj}->encode($msgstr) );
 
@@ -573,7 +575,7 @@ sub _translate_simple {
 
     $po->msgstr(
         $entry_ref->{encode_obj}->encode(
-            $self->translate_any_msgid( $entry_ref->{msgid} ),
+            $self->translate_any_msgid( $entry_ref->{msgid}, 1 ),
         ),
     );
 
@@ -657,15 +659,31 @@ sub with_lines {
         split m{ \n }xms, $msgid;
 }
 
+sub _bytes_max_fail_message {
+    my ( $self, $msgid ) = @_;
+
+    $self->bytes_max
+        or return 0;
+    my $msgid_bytes = length encode_utf8($msgid);
+
+    return $msgid_bytes > $self->bytes_max
+        ? do {
+            ( my $msgid_short = $msgid ) =~ s{ \A ( .{80} ) .* \z }{$1 ...}xms;
+            "Byte limit exceeded, $msgid_bytes bytes at: $msgid_short";
+        }
+        : q{};
+}
+
 sub _translate_paragraph {
     my ( $self, $msgid ) = @_;
 
-    my $msgid_bytes = $self->bytes_max && length encode_utf8($msgid);
-
-    return +( $msgid_bytes && $msgid_bytes > $self->bytes_max )
+    return $self->_bytes_max_fail_message($msgid)
         ? $self->with_lines(
             $msgid,
             sub {
+                my $fail_message = $self->_bytes_max_fail_message($_);
+                $fail_message
+                    and die $fail_message, "\n";
                 my $msgstr = $self->translate_text($_);
                 $self->_item_translation_count_increment;
                 $msgstr =~ s{ [\x{0}\x{4}] }{}xmsg; # because of mo file conflicts
@@ -685,31 +703,40 @@ sub _translate_paragraph {
 }
 
 sub translate_any_msgid {
-    my ( $self, $msgid ) = @_;
+    my ( $self, $msgid, $is_called_indirectly ) = @_;
 
     $self->error
-        and die 'API error: ', $self->error, "\n";
+        and $is_called_indirectly
+            ? ( die $self->error, "\n" )
+            : return q{};
     if ( $self->before_translation_code ) {
         $self->before_translation_code->($self, $msgid)
             or die "Before translation break\n";
     }
-    my $msgid_bytes = $self->bytes_max && length encode_utf8($msgid);
+    my $fail_message = $self->_bytes_max_fail_message($msgid);
     my $msgstr
         = try {
-            ( $msgid_bytes && $msgid_bytes > $self->bytes_max )
+            $fail_message
                 ? $self->with_paragraphs($msgid, sub { $self->_translate_paragraph($_) })
                 : $self->_translate_paragraph($msgid, 'is_not_paragraph');
         }
         catch {
-            m{ \QNo translation break\E }xms
-                ? q{}
-                : do {
-                    $self->_error( ( defined && length ) ? $_ : 'unknown error' );
-                    q{};
-                };
+            if ( m{ \A \QNo translation break\E \b }xms ) {
+                ;
+            }
+            elsif ( m{ \A \QByte limit exceeded, \E }xms ) {
+                chomp;
+                $self->_error($_);
+            }
+            else {
+                $self->_error("API error: $_")
+            }
+            q{};
         };
     $self->error
-        and die 'API error: ', $self->error, "\n";
+        and $is_called_indirectly
+            ? ( die $self->error, "\n" )
+            : return q{};
     $self->_translation_count_increment;
     if ( $self->after_translation_code ) {
         $self->after_translation_code->($self, $msgid, $msgstr)
@@ -735,13 +762,13 @@ __END__
 
 Locale::Utils::Autotranslator - Base class to translate automaticly
 
-$Id: Autotranslator.pm 649 2017-02-26 06:56:35Z steffenw $
+$Id: Autotranslator.pm 653 2017-06-03 20:16:11Z steffenw $
 
 $HeadURL: $
 
 =head1 VERSION
 
-1.003
+1.004
 
 =head1 SYNOPSIS
 
